@@ -50,16 +50,6 @@
 #include <curses.h>
 #endif
 
-#define DEBUG 1
-
-#ifdef __GNUC__
-#ifdef DEBUG
-#define DBG(txt, args... ) fprintf(stderr, txt , ##args )
-#else
-#define DBG(txt, args... ) /* nothing */
-#endif
-#endif
-
 enum {
   ATOM_PIXMAP = 0,
   ATOM_WINDOW,
@@ -70,6 +60,7 @@ enum {
   ATOM_COLORMAP_ENTRY,
   ATOM_PASSIVE_GRAB,
   ATOM_CURSOR,
+  ATOM_SHMSEG,
   ATOM_NET_CLIENT_LIST,
   ATOM_NET_WM_PID,
   ATOM_NET_WM_NAME,
@@ -77,7 +68,7 @@ enum {
   ATOM_COUNT
 };
 
-static char *AtomNames[] =
+static const char *AtomNames[] =
   {
     "PIXMAP",
     "WINDOW",
@@ -88,6 +79,7 @@ static char *AtomNames[] =
     "COLORMAP ENTRY",
     "PASSIVE GRAB",
     "CURSOR",
+    "ShmSeg",
     "_NET_CLIENT_LIST",
     "_NET_WM_PID",
     "_NET_WM_NAME",
@@ -112,6 +104,7 @@ typedef struct XResTopClient
   int            n_colormaps;
   int            n_passive_grabs;
   int            n_cursors;
+  int            n_shmsegs;
   int            n_other;
 
 } XResTopClient;
@@ -167,32 +160,7 @@ untrap_errors(void)
 
 /* Misc util funcs */
 
-pid_t
-window_get_pid(XResTopApp *app, Window win)
-{
-  Atom  type;
-  unsigned long  bytes_after, n_items;
-  long *data = NULL;
-  pid_t result = -1;
-  int   format;
-
-  if (XGetWindowProperty (app->dpy, win, 
-			  app->atoms[ATOM_NET_WM_PID],
-			  0, 2L,
-			  False, XA_CARDINAL,
-			  &type, &format, &n_items,
-			  &bytes_after, (unsigned char **)&data) == Success
-      && n_items && data != NULL)
-    {
-      result = *data;
-    }
-
-  if (data) XFree(data);
-
-  return result;
-}
-
-char*
+static char*
 window_get_utf8_name(XResTopApp *app, Window win)
 {
   Atom type;
@@ -209,7 +177,7 @@ window_get_utf8_name(XResTopApp *app, Window win)
 
   if (result != Success || str == NULL)
     {
-      if (str) XFree (str);
+      XFree (str);
       return NULL;
     }
 
@@ -225,7 +193,7 @@ window_get_utf8_name(XResTopApp *app, Window win)
 }
 
 
-void
+static void
 nice_bytes(char *target, int target_size, unsigned long bytes)
 {
   char prefix = 'B';
@@ -248,7 +216,7 @@ nice_bytes(char *target, int target_size, unsigned long bytes)
   snprintf(target, target_size, "%li%c", value, prefix);
 }
 
-void 
+static void 
 usage(char *progname)
 {
   fprintf(stderr, 
@@ -265,23 +233,22 @@ usage(char *progname)
 
 /* Client struct stuff */
 
-XResTopClient*
+static XResTopClient*
 xrestop_client_new(XResTopApp *app)
 {
   XResTopClient *client = NULL;
 
-  client = malloc(sizeof(XResTopClient));
-  memset(client, 0, sizeof(XResTopClient));
+  client = calloc(1,sizeof(XResTopClient));
 
   client->pid = -1;
 
   return client;
 }
 
-void
+static void
 xrestop_client_free(XResTopClient *client)
 {
-  if (client->identifier) XFree (client->identifier);
+  XFree (client->identifier);
   free(client);
 }
 
@@ -336,7 +303,7 @@ recurse_win_tree(XResTopApp *app, XResTopClient *client, Window win_top)
 {
   Window       *children, dummy;
   unsigned int  nchildren;
-  int           i;
+  unsigned int  i;
   XID           w = 0;
   Status        qtres;
   
@@ -363,12 +330,12 @@ recurse_win_tree(XResTopApp *app, XResTopClient *client, Window win_top)
 	break;
     }
 
-  if (children) XFree ((char *)children);
+  XFree ((char *)children);
 
   return w;
 }
 
-void 
+static void 
 xrestop_client_get_info(XResTopApp *app, XResTopClient *client)  
 {
   Window found = None;
@@ -390,23 +357,27 @@ xrestop_client_get_info(XResTopApp *app, XResTopClient *client)
 
   found = recurse_win_tree(app, client, app->win_root);
 
-  if (found)
-    {
-       client->pid = window_get_pid(app, found);
-    }
-  else
+  if (!found)
     {
       client->identifier = strdup("<unknown>");
     }
 }
 
-void
+static void
 xrestop_client_get_stats(XResTopApp *app, XResTopClient *client)
 {
   int               j = 0;
   XResType         *types = NULL;
   int               n_types;
 
+  XResClientIdSpec client_specs = {
+	  .client = client->resource_base,
+	  .mask = XRES_CLIENT_ID_PID_MASK
+  };
+
+  long num_ids;
+  XResClientIdValue *client_ids = NULL;
+
   trap_errors();
   
   XResQueryClientResources (app->dpy, client->resource_base, &n_types, &types);
@@ -414,6 +385,8 @@ xrestop_client_get_stats(XResTopApp *app, XResTopClient *client)
   XResQueryClientPixmapBytes (app->dpy, client->resource_base, 
 			      &client->pixmap_bytes);
   
+  XResQueryClientIds (app->dpy, 1, &client_specs, &num_ids, &client_ids);
+
   if (untrap_errors())
     {
       app->n_xerrors++;
@@ -422,7 +395,7 @@ xrestop_client_get_stats(XResTopApp *app, XResTopClient *client)
   
   for (j=0; j < n_types; j++)
     {
-      int this_type = types[j].resource_type;
+      Atom this_type = types[j].resource_type;
       
       if (this_type == app->atoms[ATOM_PIXMAP])
 	client->n_pixmaps += types[j].count;
@@ -442,9 +415,20 @@ xrestop_client_get_stats(XResTopApp *app, XResTopClient *client)
 	client->n_passive_grabs += types[j].count;
       else if (this_type == app->atoms[ATOM_CURSOR])
 	client->n_cursors += types[j].count;
+      else if (this_type == app->atoms[ATOM_SHMSEG])
+	client->n_shmsegs += types[j].count;
       else client->n_other += types[j].count;
     }
 
+  for (j = 0; j < num_ids; j++)
+  {
+    if (client_ids[j].spec.mask == XRES_CLIENT_ID_PID_MASK && client_ids[j].length == sizeof(uint32_t)) {
+      uint32_t	pid;
+      memcpy(&pid, client_ids[j].value, sizeof(uint32_t));
+      client->pid = pid;
+    }
+  }
+
   /* All approx currently - same as gnome system monitor */
    client->other_bytes += client->n_windows * 24;
    client->other_bytes += client->n_gcs * 24;
@@ -454,15 +438,17 @@ xrestop_client_get_stats(XResTopApp *app, XResTopClient *client)
    client->other_bytes += client->n_colormaps * 24;
    client->other_bytes += client->n_passive_grabs * 24;
    client->other_bytes += client->n_cursors * 24;
+   client->other_bytes += client->n_shmsegs * 24;
    client->other_bytes += client->n_other * 24;
   
  cleanup:
-   if (types) XFree(types);
+   XFree(types);
+   XFree(client_ids);
 
    return;
 }
 
-void
+static void
 xrestop_populate_client_data(XResTopApp *app)
 {
   int         i;
@@ -495,17 +481,17 @@ xrestop_populate_client_data(XResTopApp *app)
 
  cleanup:
 
-  if (clients) XFree(clients);
+  XFree(clients);
 }
 
-void
+static void
 xrestop_display(XResTopApp *app)
 {
   int  i;
-  char pretty_pixmap_bytes[16] = { 0 };
-  char pretty_other_bytes[16]  = { 0 };
-  char pretty_total_bytes[16]  = { 0 };
-  char pretty_pid[16]          = { 0 };
+  char pretty_pixmap_bytes[20] = { 0 };
+  char pretty_other_bytes[20]  = { 0 };
+  char pretty_total_bytes[20]  = { 0 };
+  char pretty_pid[20]          = { 0 };
 
   if (!app->want_batch_mode)
     {
@@ -519,9 +505,9 @@ xrestop_display(XResTopApp *app)
 	  total_other_bytes += app->clients[i]->other_bytes;
 	}
 
-      nice_bytes(pretty_pixmap_bytes, 16, total_pixmap_bytes);
-      nice_bytes(pretty_other_bytes, 16, total_other_bytes);
-      nice_bytes(pretty_total_bytes, 16, 
+      nice_bytes(pretty_pixmap_bytes, sizeof(pretty_pixmap_bytes), total_pixmap_bytes);
+      nice_bytes(pretty_other_bytes, sizeof(pretty_other_bytes), total_other_bytes);
+      nice_bytes(pretty_total_bytes, sizeof(pretty_total_bytes),
 		 total_pixmap_bytes + total_other_bytes);
 
       /* Curses rendering  */
@@ -540,16 +526,18 @@ xrestop_display(XResTopApp *app)
 
       attron(A_BOLD|A_REVERSE);
 
-      mvprintw(4, 0, "res-base Wins  GCs Fnts Pxms Misc   Pxm mem  Other   Total   PID Identifier    ");
+      mvprintw(4, 0, "res-base Wins  GCs Fnts Pxms ShmS Misc   Pxm mem  Other   Total   PID Identifier    ");
       
       attroff(A_BOLD|A_REVERSE);
     }
 
   for (i=0; i<app->n_clients; i++)
     {
-      nice_bytes(pretty_pixmap_bytes, 16, app->clients[i]->pixmap_bytes);
-      nice_bytes(pretty_other_bytes, 16, app->clients[i]->other_bytes);
-      nice_bytes(pretty_total_bytes, 16, 
+       nice_bytes(pretty_pixmap_bytes, sizeof(pretty_pixmap_bytes),
+                 app->clients[i]->pixmap_bytes);
+       nice_bytes(pretty_other_bytes, sizeof(pretty_other_bytes),
+                 app->clients[i]->other_bytes);
+       nice_bytes(pretty_total_bytes, sizeof(pretty_total_bytes),
 		 app->clients[i]->pixmap_bytes + app->clients[i]->other_bytes);
 
       if (app->clients[i]->pid > -1)
@@ -560,13 +548,14 @@ xrestop_display(XResTopApp *app)
 
       if (!app->want_batch_mode)
 	{
-	  mvprintw(i+5, 0, "%.7x  %4d %4d %4d %4d %4d   %7s %7s %7s %5s %s", 
+	  mvprintw(i+5, 0, "%.7x  %4d %4d %4d %4d %4d %4d   %7s %7s %7s %5s %s",
 		   
 		   app->clients[i]->resource_base, 
 		   app->clients[i]->n_windows, 
 		   app->clients[i]->n_gcs, 
 		   app->clients[i]->n_fonts,
 		   app->clients[i]->n_pixmaps,  
+		   app->clients[i]->n_shmsegs,
 		   
 		   app->clients[i]->n_pictures 
 		   + app->clients[i]->n_glyphsets
@@ -587,8 +576,8 @@ xrestop_display(XResTopApp *app)
       else
 	{
 	  printf("%i - %s ( PID:%s ):\n"
-		 "\tres_base      : ox%lx\n"
-		 "\tres_mask      : ox%lx\n"
+		 "\tres_base      : %#lx\n"
+		 "\tres_mask      : %#lx\n"
 		 "\twindows       : %d\n"
 		 "\tGCs           : %d\n"
 		 "\tfonts         : %d\n"
@@ -598,6 +587,7 @@ xrestop_display(XResTopApp *app)
 		 "\tcolormaps     : %d\n"
 		 "\tpassive grabs : %d\n"
 		 "\tcursors       : %d\n"
+		 "\tshm segments  : %d\n"
 		 "\tunknowns      : %d\n"
 		 "\tpixmap bytes  : %ld\n"
 		 "\tother bytes   : ~%ld\n"
@@ -616,6 +606,7 @@ xrestop_display(XResTopApp *app)
 		 app->clients[i]->n_colormaps,
 		 app->clients[i]->n_passive_grabs,
 		 app->clients[i]->n_cursors,
+		 app->clients[i]->n_shmsegs,
 		 app->clients[i]->n_other, 
 		 app->clients[i]->pixmap_bytes,
 		 app->clients[i]->other_bytes,
@@ -627,7 +618,7 @@ xrestop_display(XResTopApp *app)
     refresh();
 }
 
-int 
+static int 
 xrestop_sort_compare(const void *a, const void *b)
 {
   XResTopClient *c1 = *(XResTopClient **)a;
@@ -640,7 +631,7 @@ xrestop_sort_compare(const void *a, const void *b)
 }
 
 
-void
+static void
 xrestop_sort(XResTopApp *app)
 {
   qsort((void *)app->clients, app->n_clients, sizeof(app->clients[0]), xrestop_sort_compare);
@@ -652,8 +643,7 @@ main(int argc, char **argv)
   int      i, event, error, major, minor;
   XResTopApp *app = NULL;
 
-  app = malloc(sizeof(XResTopApp));
-  memset(app, 0, sizeof(XResTopApp));
+  app = calloc(1,sizeof(XResTopApp));
 
   app->delay = 2;
 
@@ -704,7 +694,7 @@ main(int argc, char **argv)
   app->screen = DefaultScreen(app->dpy);
   app->win_root = RootWindow(app->dpy, app->screen); 
     
-  XInternAtoms (app->dpy, AtomNames, ATOM_COUNT,False, app->atoms);
+  XInternAtoms (app->dpy, (char **)AtomNames, ATOM_COUNT,False, app->atoms);
 
   if(!XResQueryExtension(app->dpy, &event, &error)) {
     fprintf(stderr, "%s: XResQueryExtension failed. Display Missing XRes extension ?\n", argv[0]);
